package org.xdv.clx.expr;

import org.xdv.clx.exec.ClxExecutionContext;
import org.xdv.clx.exec.ClxExecutionPoint;
import org.xdv.common.XdvValidationException;

/**
 * NOT - CLX element implementation.
 * 
 * @see <a href="http://www.clixml.org/clix/1.0/#not">CLiXML - clx:not</a>
 * @see org.xdv.clx.base.ClxConstants#TAG_NOT
 * @author Tomasz Kokoszka
 */
public class ClxNot implements ClxFormula {
    
    private final ClxFormula formula;

    /**
     * Creates instance.
     * @param formula sub-formula
     */
    public ClxNot(ClxFormula formula) {
        super();
        this.formula = formula;
    }

    public ClxFormula getFormula() {
        return formula;
    }        

    public boolean executeFormula(ClxExecutionContext cntx) 
    throws XdvValidationException {
        return ! this.getFormula().executeFormula(cntx);
    }
    
    public boolean executeFormula(ClxExecutionContext cntx, ClxExecutionPoint frame) 
    throws XdvValidationException {
        boolean b = ! this.getFormula().executeFormula(cntx, frame.getNext1());
        frame.setFinished();
        return b;
    }
    
    public String toString() {
        return "!(" + formula + ")";
    }
        
}
