package org.xdv.clx.expr;

import org.xdv.clx.exec.ClxExecutionContext;
import org.xdv.common.XdvValidationException;
import org.xdv.xpath.XPathExpression;
import org.xdv.xpath.XPathValue;
import org.xdv.xpath.XPathValueNodeSet;

/**
 * SAME - CLX element implementation.
 * 
 * @see <a href="http://www.clixml.org/clix/1.0/#same">CLiXML - clx:same</a>
 * @see org.xdv.clx.base.ClxConstants#TAG_SAME
 * @author Tomasz Kokoszka
 */
public class ClxSame extends ClxFormulaBinaryPredicate {

    /**
     * Creates instance.
     * @param op1 first expression
     * @param op2 secound expression
     */
    public ClxSame(XPathExpression op1, XPathExpression op2) {
        super(op1, op2);
    }
        
    /**
     * Returns a XML node from value.
     * @param value XPath value
     * @return XML node; !=null
     * @throws XdvValidationException value doesn't store a XML node
     */
    private Object getNode(XPathValue value)
    throws XdvValidationException {
        if (value.getType() == XPathValue.TYPE_NODESET) {
            final XPathValueNodeSet nodeset = (XPathValueNodeSet) value;
            if (nodeset.getLength() != 1) {
                throw new XdvValidationException(
                        "Too many nodes - xpath must return a single node for predicate same");
            }
            return nodeset.getFirstNode();
        }
        throw new XdvValidationException(
                "Node is missing - xpath must return a single node for predicate same ");
    }

    public boolean executeFormula(ClxExecutionContext cntx) 
    throws XdvValidationException {
        Object n1 = getNode(executeXPath(getOp1(), cntx));
        Object n2 = getNode(executeXPath(getOp2(), cntx));
        return cntx.getXPathEngine().isSameNode(n1, n2);
    }
    
    public String toString() {
        return getOp1() + "===" + getOp2();
    }
    
}
