package org.xdv.xpath;

/**
 * XPath error - invalid object type.
 * <p>
 * It is used to report a problem when passed parameter cannot be casted to type required by 
 * underlying XPath implementation.
 * <br>
 * For instnace see {@link XPathEngine#createExecutionUnit(Object)}.
 * </p>
 * <p>
 * It is a simple extension of {@link java.lang.Error}. It provides two additional constructors
 * to easily generate proper error message.
 * For detailed description about constructors please look directly to 
 * superclass ({@link Error}).
 * </p> 
 *
 * @author Tomasz Kokoszka
 */
public class XPathInvalidTypeError extends Error {

    /**
     * Constructs a new error with <code>null</code> as its detail message.
     * @see Error#Error()   
     */    
    public XPathInvalidTypeError() {
        super();
    }
    
    /**
     * Constructs a new error with detail message saying that a different type of object was 
     * expected (plus full name of expected and given type/class).
     * @param givenObject given object, it is not an instance of expected type; can be null
     * @param expected expected object type (class)
     */
    public XPathInvalidTypeError(Object givenObject, Class expected) {
        this("Invalid object class (" + 
             ((givenObject == null)?"null":givenObject.getClass().getName()) +
             "), " + expected.getName() + " required");
    }

    /**
     * Constructs a new error with detail message saying that a different type of object was 
     * expected (plus full name of expected and given class/type).
     * @param given type/class of given object; can be null
     * @param expected expected object type/class
     */    
    public XPathInvalidTypeError(Class given, Class expected) {
        this("Invalid object class (" + given.getName() + "), " + expected.getName() + " required");
    }

    /**
     * Constructs a new error with the specified detail message and cause.
     * @param message the detail message; can be null
     * @param cause the cause; can be null
     * @see Error#Error(java.lang.String, java.lang.Throwable)
     */
    public XPathInvalidTypeError(String message, Throwable cause) {
        super(message, cause);
    }

    /**
     * Constructs a new error with the specified detail message. 
     * @param message the detail message; can be null
     * @see Error#Error(java.lang.String)
     */
    public XPathInvalidTypeError(String message) {
        super(message);
    }

    /**
     * Constructs a new error with the specified cause and a detail
     * message of <tt>(cause==null ? null : cause.toString())</tt>.
     * @param cause the cause; can be null
     * @see Error#Error(java.lang.Throwable)
     */
    public XPathInvalidTypeError(Throwable cause) {
        super(cause);
    }        

}
